#include <graphics>
#include <stdio.h>
#include <conio.h>

#include <stdinc.c>
#include <keycodes.c>

#define LimX 320
#define LimY 192
#define CenterX LimX / 2
#define CenterY LimY / 2
#define PlayTo 15
#define SlowFactor 800

typedef struct {
  unsigned x, y;
  byte radius, color;
  small addx, addy;
} ball;

typedef struct {
  byte color, score;
  unsigned x, y;
  small addy;
} pad;

#define PadHeight 24
#define PadWidth 4

char pause ();
void pad_draw (pad p, boolean b);
void pad_move (pad *p);
void ball_draw (ball b, boolean bool);
void ball_move (ball *b, boolean mute);
void game_init (pad *p1, pad *p2, boolean *b, byte *h, boolean *m);
unsigned strlen (char *s);
boolean pad_ball_collide (pad *p1, pad *p2, ball *b, boolean mute);
void init_pos (pad *p1, pad *p2, ball *b);
char *str (unsigned i);
unsigned unsignedlen (unsigned u);
boolean hit_esc (int vmode);
unsigned strwidth (char *s);
void pen (byte b1, byte b2);
void ai_pad (pad *p, ball, byte hardness);
unsigned rnd (unsigned u);
void game_won (byte b, int vmode);

void main () {
  int vmode = getvmode ();
  struct vconfig screen;
  pad p1, p2;
  ball b;
  char inkey = 0;
  boolean onePlyr = false, mute;
  byte hardness;
  unsigned wait;

  srand (wait * hardness);
  setvmode (EGA_320);  
  getvconfig (&screen);

  game_init (&p1, &p2, &onePlyr, &hardness, &mute);

  p1.score = 0;
  p2.score = 0;
  b.radius = 12;
  b.color = 15;
  b.addx = 4; b.addy = 4;

  init_pos (&p1, &p2, &b);

  for (;;) {
    pad_move (&p1);
    pad_move (&p2);
    ball_move (&b, mute);
    if (pad_ball_collide (&p1, &p2, &b, mute)) {
      if ((p1.score >= PlayTo or p2.score >= PlayTo) and (abs (p1.score - p2.score) >= 2)) {
        if (p1.score > p2.score) 
          game_won (1, vmode);
        else 
          game_won (2, vmode);
        break;
      }
      
      clrscrn2 (0);
      init_pos (&p1, &p2, &b);
    }

    if (onePlyr) ai_pad (&p2, b, hardness);
    //delay (Speed);
    for (wait = 0; wait < (26 - hardness) * SlowFactor; wait++);
    if (!kbhit ()) continue;

    inkey = getch ();
    if (!inkey) { 
      inkey = getch ();
      switch (inkey) {
        case Up :
          p1.addy = -4;
          break;
        case Down :
          p1.addy = 4;
      }
    } else {
      switch (inkey) {
        case '8' :
          p1.addy = -4;
          break;
        case '2' :
          p1.addy = 4;
          break;
        case 32 :
          mute = !mute;
          break;
        case 'W' :
        case 'w' :
          if (!onePlyr) p2.addy = -4;
          break;
        case 'S' : 
        case 's' :
          if (!onePlyr) p2.addy = 4;
        }
    }

    if (inkey == 27)
      if (hit_esc (vmode))
        break;
  };
  
  setvmode (vmode);
}

char pause () {
  while (!kbhit ());
  return getch ();
}

void pad_draw (pad p, boolean b) {
  move_to (p.x, p.y);
  if (b) 
    pen_color (p.color);
  else
    pen_color (0);

  flood (PadWidth, PadHeight);
}

void pad_move (pad *p) {
  pad_draw (*p, false);

  p->y += p->addy;
  if (p->y == 0 or p->y == LimY - PadHeight)
    p->y -= p->addy;
  
  pad_draw (*p, true);
}

void ball_draw (ball b, boolean bool) {
  byte i;

  move_to (b.x, b.y);
  if (bool) 
    circle (b.radius, b.color);
  else 
    circle (b.radius, 0);
}

void ball_move (ball *b, boolean mute) {
  ball_draw (*b, false);

  b->x += b->addx;
  b->y += b->addy;
  if (b->y == b->radius or b->y == LimY - b->radius) {
    b->addy = -b->addy;
    b->y += b->addy;
    if (!mute) sound (60, 1);
  }

  ball_draw (*b, true);
}

void game_init (pad *p1, pad *p2, boolean *b, byte *h, boolean *m) {
  char msg[30];
  byte option = 1;
  char inkey = 0;

  clrscrn2 (0);
  
  do {
    pen (option, 1);
    strcpy (msg, "One Player Game");
    move_to (CenterX - strwidth (msg), CenterY - 16);
    plots (msg);
  
    pen (option, 2);
    strcpy (msg, "Two Player Game");
    move_to (CenterX - strwidth (msg), CenterY);
    plots (msg);
  
    inkey = pause ();
    if (inkey == Up or inkey == '8') option--;
    if (inkey == Down or inkey == '2') option++;
    if (option == 0) option = 2;
    if (option == 3) option = 1;
  } until (inkey == 13);
  *b = !(option - 1);

  clrscrn2 (0);
  option = 1;
  pen_color (15);
  if (*b) {
    strcpy (msg, "Select Color:");    
    move_to (CenterX - strwidth (msg), CenterY);
    plots (msg);  
    move_to (CenterX + strwidth (msg), CenterY);  
  } else {
    strcpy (msg, "Select Color Player 1:");
    move_to (CenterX - strwidth (msg), CenterY - 8);  
    plots (msg);
    move_to (CenterX + strwidth (msg), CenterY - 8);
  }

  do {
    pen_color (option);
    flood (4, 8);
    inkey = pause ();
    if (inkey == Up or inkey == '8') option++;
    if (inkey == Down or inkey == '2') option--;
    if (option == 0) option = 15;
    if (option == 16) option = 1;
  } until (inkey == 13);
  p1->color = option;

  if (!*b) {
    pen_color (15);
    strcpy (msg, "Select Color Player 2:");
    move_to (CenterX - strwidth (msg), CenterY + 8);
    plots (msg);
    move_to (CenterX + strwidth (msg), CenterY + 8);
    option = 2;
    do {
      pen_color (option);
      flood (4, 8);
      inkey = pause ();
      if (inkey == 'w' or inkey == 'W') option++;
      if (inkey == 's' or inkey == 'S') option--;
      if (option == 0) option = 15;
      if (option == 16) option = 1;
    } until (inkey == 13);
    p2->color = option;
  } else
    p2->color = 10;
  
  clrscrn2 (0);
  
  if (*b)
    strcpy (msg, "Select Difficulty (1-5): ");
  else
    strcpy (msg, "Select Game Speeed (1-5): ");
  
  move_to (CenterX - strwidth (msg), CenterY);
  pen_color (15);
  plots (msg);
  pen_color (14);
  *h = '3';
  do {
    move_to (CenterX + strwidth (msg) + 1, CenterY);
    plotch (*h);
    inkey = pause ();
    if (inkey == Up or inkey == '8') ++*h;
    if (inkey == Down or inkey == '2') --*h;
    if (*h == '0') *h = '5';
    if (*h == '6') *h = '1';
  } until (inkey == 13);
  *h = 5 * (*h - 48);
    
  clrscrn2 (0);

  pen_color (15);
  strcpy (msg, "Sound? ");
  move_to (CenterX - strwidth (msg), CenterY);
  plots (msg);
  *m = false;
  pen_color (14);
  do {
    move_to (CenterX + strwidth (msg), CenterY);    
    if (*m)
      plots ("Off");
    else
      plots ("On ");
    inkey = pause ();    
    if (inkey == Up or inkey == Down or inkey == '2' or inkey == '8') *m = !*m;
  } until (inkey == 13);

}

unsigned strlen (char *s) {
  unsigned len;
  for (len = 0; *(s + len); len++);
  return len;
}

boolean pad_ball_collide (pad *p1, pad *p2, ball *b, boolean mute) {
  if (b->x == b->radius) {
    p2->score++;
    return true;
  }

  if (b->x == LimX - b->radius) { 
    p1->score++;
    return true;
  }

  if ((b->x - b->radius == p1->x + PadWidth and b->y + b->radius >= p1->y and b->y - b->radius <= p1->y + PadHeight)
      or (b->x + b->radius == p2->x and b->y + b->radius >= p2->y and b->y - b->radius <= p2->y + PadHeight)) {
    b->addx = -b->addx;
    if (!mute) sound (80, 2);
  }

  return false;
}

void init_pos (pad *p1, pad *p2, ball *b) {
  char msg[30];

  p1->x = 4; p1->y = CenterY;
  p2->x = LimX - 4 - PadWidth; p2->y = CenterY;
  p1->addy = 0; p2->addy = 0;
  b->x = CenterX;
  b->y = CenterY;
  b->addx = -b->addx;
  b->addy = -b->addy;
  if (rnd (2))
    if (rnd (1))
      b->y += b->addy;
    else
      b->y -= b->addy;

  pen_color (p1->color);
  move_to (0, LimY);
  strcpy (msg, "Player 1 Score = ");
  strcat (msg, str (p1->score));
  plots (msg);
  
  pen_color (p2->color);
  strcpy (msg, "Player 2 Score = ");
  strcat (msg, str (p2->score));
  move_to (LimX - strlen (msg) * 8, LimY);
  plots (msg);

  pad_draw (*p1, true);
  pad_draw (*p2, true);
  ball_draw (*b, true);  
  
  pen_color (14);
  strcpy (msg, "Press any key to begin!");
  move_to (CenterX - strwidth (msg), CenterY);
  plots (msg);

  pause ();

  pen_color (0);
  move_to (CenterX - strwidth (msg), CenterY);
  plots (msg);
  
  ball_draw (*b, true);
}

char *str (unsigned i) {
  unsigned j;
  unsigned length = unsignedlen (i) - 1;
  char *string;

  if (i == 0) return "0";

  string += length;
  string = 0;
  
  while (i >= 1) {
    j = i % 10;
    string--;
    *string = j + 48;
    i /= 10;
  }

  return string;
}

unsigned unsignedlen (unsigned u) {
  unsigned length;
  if (u == 0) return 1;
  for (length = 0; u >= 1; u /= 10, length++);
  return length;
}

boolean hit_esc (int vmode) {
  char msg[30], inkey = 0;
  byte option = 1;

  setapage (1);
  setvpage (1);
    
  for (;;) {
    pen (option, 1);
    strcpy (msg, "Exit Pong");
    move_to (CenterX - strwidth (msg), CenterY - 16);
    plots (msg);

    pen (option, 2);
    strcpy (msg, "Start Over");
    move_to (CenterX - strwidth (msg), CenterY);
    plots (msg);

    pen (option, 3);
    strcpy (msg, "Continue Game");
    move_to (CenterX - strwidth (msg), CenterY + 16);
    plots (msg);
  
    inkey = pause ();

    if (inkey == Up) option--;
    if (inkey == Down) option++;

    if (option == 0) option = 3;
    if (option == 4) option = 1;
  
    if (inkey == 13 or inkey == 27) {
      setapage (0);
      setvpage (0);
      
      switch (option) {
        case 1 : 
          return true;
        case 2 : 
          setvmode (vmode);
          main ();
          return true;
        case 3 :
          return false;
      }
    }
  }
}

unsigned strwidth (char *s) {
  return strlen (s) * 4;
}

void pen (byte b1, byte b2) {
  if (b1 == b2)
    pen_color (14);
  else
    pen_color (15);
}

void ai_pad (pad *p, ball b, byte hardness) {
  if (b.x < CenterX) return;

  if (b.y > p->y) 
    p->addy = 4;
  else if (b.y < p->y)
    p->addy = -4;
  else
    p->addy = b.addy;

  if (!rnd (hardness))
    p->addy = -p->addy;
}

unsigned rnd (unsigned u) {
  return rand () % u;
}

void game_won (byte b, int vmode) {
  char msg[30], inkey;
  boolean again = true;

  clrscrn2 (0);
  if (b == 1)
    strcpy (msg, "Congradulations Player One!!!");
  else
    strcpy (msg, "Congradulations Player Two!!!");

  pen_color (15);  
  move_to (CenterX - strwidth (msg), CenterY);
  plots (msg);

  sleep (2);

  clrscrn2 (0);

  strcpy (msg, "Do you wish to play again? ");
  move_to (CenterX - strwidth (msg), CenterY);
  plots (msg);
  pen_color (14);  
  
  do {
    move_to (CenterX + strwidth (msg), CenterY);
    if (again) 
      plots ("Yes");
    else
      plots ("No ");
  
    inkey = pause ();
    if (inkey == Up or inkey == Down) again = !again;
  } until (inkey == 13);
  
  if (again) {
    setvmode (vmode);
    main ();
  } else {
    clrscrn2 (0);
    pen_color (10);
    strcpy (msg, "Pong");
    move_to (CenterX - strwidth (msg), CenterY - 16);
    plots (msg);
    strcpy (msg, "Game by: Jimmy Reumuth");
    move_to (CenterX - strwidth (msg), CenterY);
    plots (msg);
    strcpy (msg, "June 25, 1998");
    move_to (CenterX - strwidth (msg), CenterY + 16);
    plots (msg);

    pause ();
  }
}
